/* mySQLQueryResult.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#ifdef USE_MYSQL

#include "mySQLQueryResult.h"
#include "mySQLDatabase.h"
#include "dataCell.h"

MySQLQueryResult::
MySQLQueryResult(MySQLDatabase &db) : QueryResultBase(db) { 
  _has_error = false;
  _my_result = NULL;
  _columns = NULL;
  _num_columns = 0;
  _current_row_index = -1;
}

MySQLQueryResult::
~MySQLQueryResult() {
  set_my_result(NULL);
}

bool MySQLQueryResult::
success() const {
  return !_has_error;
}

string MySQLQueryResult::
get_error_message() const {
  return _error_message;
}

int MySQLQueryResult::
get_num_columns() const {
  return _num_columns;
}

string MySQLQueryResult::
get_column_name(int c) const {
  assert(c >= 0 && c < _num_columns);
  return _columns[c].name;
}

DataCell::Type MySQLQueryResult::
get_column_type(int c) const {
  switch (_columns[c].type) {
  case FIELD_TYPE_NULL:
    return DataCell::DC_null;

  case FIELD_TYPE_TINY:
  case FIELD_TYPE_SHORT:
  case FIELD_TYPE_LONG:
  case FIELD_TYPE_INT24:
  case FIELD_TYPE_LONGLONG:
  case FIELD_TYPE_YEAR:
    return DataCell::DC_int;

  case FIELD_TYPE_DECIMAL:
  case FIELD_TYPE_FLOAT:
  case FIELD_TYPE_DOUBLE:
    return DataCell::DC_float;

  case FIELD_TYPE_TIMESTAMP:
  case FIELD_TYPE_DATE:
  case FIELD_TYPE_TIME:
  case FIELD_TYPE_DATETIME:
    return DataCell::DC_timestamp;

  case FIELD_TYPE_STRING:
  case FIELD_TYPE_BLOB:
  default:
    return DataCell::DC_string;
  }
}

int MySQLQueryResult::
get_num_rows() const {
  return (_my_result == NULL) ? 0 : mysql_num_rows(_my_result);
}

DataCell MySQLQueryResult::
get_cell(int r, int c) const {
  assert(_my_result != NULL);

  // A bit of const-fiddling, so we can update the cached row value
  // without apparently changing the value of this structure.
  ((MySQLQueryResult *)this)->set_row(r);

  if (_current_row[c] == NULL) {
    return DataCell();
  } else {
    return DataCell(_current_row[c], get_column_type(c));
  }
}


void MySQLQueryResult::
set_my_result(MYSQL_RES *result) {
  if (_my_result != NULL) {
    mysql_free_result(_my_result);
    _my_result = NULL;
  }
  _my_result = result;
  _has_error = false;
  _current_row_index = -1;
  if (_my_result == NULL) {
    _columns = NULL;
    _num_columns = 0;
  } else {
    _columns = mysql_fetch_fields(_my_result);
    _num_columns = mysql_num_fields(_my_result);
  }
}

void MySQLQueryResult::
set_error_message(const string &msg) {
  _error_message = msg;
  _has_error = true;
}


void MySQLQueryResult::
set_row(int row) {
  assert(_my_result != NULL);
  if (row != _current_row_index) {
    mysql_data_seek(_my_result, row);
    _current_row = mysql_fetch_row(_my_result);
  }
}

#endif // USE_MYSQL
