/* vplaysf.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

// This is a program that uses libaudio to read a sound file (such as
// a WAV or AIFF file) and pipe it to the OSS soundcard.

// It is specifically designed to be useful as a backend to Paloma, as
// it:

// 1) does not terminate until the song is finished

// 2) reports current position in the song during playback, and

// 3) supports playing an arbitrary subset of a given file.

#include "system.h"

#include "sfPlayer.h"
#include "mainProgram.h"

#include <string>
#include <iostream.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static const string default_sound_device = "/dev/dsp";
static const int default_block_samples = 1024;

class VPlaySFMain : public MainProgram {
public:
  VPlaySFMain();

  virtual void run();

protected:
  virtual void handle_opts(const string &opt, const string &arg);
  virtual void handle_args(const Args &args);
  virtual void help();
  virtual void describe_options();
  virtual void describe_args();

  string _sound_device;
  int _block_samples;
  double _start_time;
  double _play_time;
  bool _verbose;
  bool _length_only;

  string _filename;
};

VPlaySFMain::
VPlaySFMain() : MainProgram("vplaysf") 
{
  define_opt("d", true);
  define_opt("b", true);
  define_opt("s", true);
  define_opt("p", true);
  define_opt("v", false);
  define_opt("l", false);

  _sound_device = default_sound_device;
  _block_samples = default_block_samples;
  _start_time = 0.0;
  _play_time = -1.0;
  _verbose = false;
  _length_only = false;
}

void VPlaySFMain::
run() {
  bool uses_stdin = false;
  int soundfile_fd;

  if (_filename == "-") {
    soundfile_fd = STDIN_FILENO;
    uses_stdin = true;
  } else {
    soundfile_fd = open(_filename.c_str(), O_RDONLY);
    if (soundfile_fd < 0) {
      perror(_filename.c_str());
      exit(1);
    }
  }

  SFPlayer player(soundfile_fd, _filename, _sound_device, _block_samples);
  if (player.is_valid()) {
    if (_length_only) {
      cerr << "Soundfile is " << player.get_length() << " seconds.\n";
      exit(0);
    }

    if (!uses_stdin) {
      cerr << "Playing " << _filename << "\n";
    }

    if (_start_time > 0.0) {
      if (_play_time >= 0.0) {
	cerr << "Playing sub-range " << _start_time << " to " 
	     << _start_time + _play_time << "\n";
      } else {
	cerr << "Starting playback at time " << _start_time << "\n";
      }
    } else if (_play_time >= 0.0) {
      cerr << "Playing only the first " << _play_time << " seconds.\n";
    }

    player.play(_start_time, _play_time, _verbose);
  }

  cerr << "\nDone.\n";
}

void VPlaySFMain::
handle_opts(const string &opt, const string &arg) {
  if (opt == "d") {
    _sound_device = arg;

  } else if (opt == "b") {
    _block_samples = atoi(arg.c_str());

  } else if (opt == "s") {
    _start_time = atof(arg.c_str());
  
  } else if (opt == "p") {
    _play_time = atof(arg.c_str());

  } else if (opt == "v") {
    _verbose = true;

  } else if (opt == "l") {
    _length_only = true;

  } else {
    MainProgram::handle_opts(opt, arg);
  }
}

void VPlaySFMain::
handle_args(const Args &args) {
  if (args.size() != 1) {
    usage();
    exit(1);
  }

  _filename = args[0];
}

void VPlaySFMain::
help() {
  cerr << 
    "\n"
    "This is yet another program to play sound files through the system audio\n"
    "device.  It is somewhat specialized to enable it to work well with Paloma.\n";
  MainProgram::help();
}

void VPlaySFMain::
describe_options() {
  cerr <<
    "  -d device  The audio device to access.  The default if this is unspecified\n"
    "             is " << default_sound_device << ".\n"
    "  -b num     Specify the number of samples to write to the audio device at a\n"
    "             time.  It is not clear what effect adjusting this number has on\n"
    "             the likelihood of skips during playback.  The default is " << default_block_samples << ".\n"
    "  -s secs    Start at the indicated number of seconds into the file.\n"
    "  -p secs    Play for the indicated number of seconds, and stop.\n"
    "  -v         Output an indication of progress through the file.\n"
    "  -l         Report only the length of the file in seconds, and terminate.\n"

    "\n";
}

void VPlaySFMain::
describe_args() {
  cerr <<
    " soundfile\n\n"

    "soundfile may be, at present, any WAV or AIFF formatted sound file.\n\n";
}

int
main(int argc, char *argv[]) {
  VPlaySFMain m;
  m.command_line(argc, argv);
  m.init();
  m.run();
  return (0);
}
